# DO NOT EDIT! This file is automatically generated
import typing

from commercetools._schemas._shopping_list import (
    ShoppingListDraftSchema,
    ShoppingListPagedQueryResponseSchema,
    ShoppingListSchema,
    ShoppingListUpdateSchema,
)
from commercetools.helpers import RemoveEmptyValuesMixin
from commercetools.types._shopping_list import (
    ShoppingList,
    ShoppingListDraft,
    ShoppingListPagedQueryResponse,
    ShoppingListUpdate,
    ShoppingListUpdateAction,
)
from commercetools.typing import OptionalListStr

from . import abstract, traits


class _ShoppingListQuerySchema(
    traits.ExpandableSchema,
    traits.SortableSchema,
    traits.PagingSchema,
    traits.QuerySchema,
):
    pass


class _ShoppingListUpdateSchema(traits.ExpandableSchema, traits.VersionedSchema):
    pass


class _ShoppingListDeleteSchema(
    traits.VersionedSchema, traits.ExpandableSchema, traits.DataErasureSchema
):
    pass


class ShoppingListService(abstract.AbstractService):
    """shopping-lists e.

    g. for wishlist support
    """

    def get_by_id(self, id: str, *, expand: OptionalListStr = None) -> ShoppingList:
        """Gets a shopping list by ID."""
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"shopping-lists/{id}",
            params=params,
            schema_cls=ShoppingListSchema,
        )

    def get_by_key(self, key: str, *, expand: OptionalListStr = None) -> ShoppingList:
        """Gets a shopping list by Key."""
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"shopping-lists/key={key}",
            params=params,
            schema_cls=ShoppingListSchema,
        )

    def query(
        self,
        *,
        expand: OptionalListStr = None,
        sort: OptionalListStr = None,
        limit: int = None,
        offset: int = None,
        with_total: bool = None,
        where: OptionalListStr = None,
        predicate_var: typing.Dict[str, str] = None,
    ) -> ShoppingListPagedQueryResponse:
        """shopping-lists e.g. for wishlist support
        """
        params = self._serialize_params(
            {
                "expand": expand,
                "sort": sort,
                "limit": limit,
                "offset": offset,
                "with_total": with_total,
                "where": where,
                "predicate_var": predicate_var,
            },
            _ShoppingListQuerySchema,
        )
        return self._client._get(
            endpoint="shopping-lists",
            params=params,
            schema_cls=ShoppingListPagedQueryResponseSchema,
        )

    def create(
        self, draft: ShoppingListDraft, *, expand: OptionalListStr = None
    ) -> ShoppingList:
        """shopping-lists e.g. for wishlist support
        """
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._post(
            endpoint="shopping-lists",
            params=params,
            data_object=draft,
            request_schema_cls=ShoppingListDraftSchema,
            response_schema_cls=ShoppingListSchema,
        )

    def update_by_id(
        self,
        id: str,
        version: int,
        actions: typing.List[ShoppingListUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> ShoppingList:
        params = self._serialize_params({"expand": expand}, _ShoppingListUpdateSchema)
        update_action = ShoppingListUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"shopping-lists/{id}",
            params=params,
            data_object=update_action,
            request_schema_cls=ShoppingListUpdateSchema,
            response_schema_cls=ShoppingListSchema,
            force_update=force_update,
        )

    def update_by_key(
        self,
        key: str,
        version: int,
        actions: typing.List[ShoppingListUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> ShoppingList:
        """Update a shopping list found by its Key."""
        params = self._serialize_params({"expand": expand}, _ShoppingListUpdateSchema)
        update_action = ShoppingListUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"shopping-lists/key={key}",
            params=params,
            data_object=update_action,
            request_schema_cls=ShoppingListUpdateSchema,
            response_schema_cls=ShoppingListSchema,
            force_update=force_update,
        )

    def delete_by_id(
        self,
        id: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        data_erasure: bool = None,
        force_delete: bool = False,
    ) -> ShoppingList:
        params = self._serialize_params(
            {"version": version, "expand": expand, "data_erasure": data_erasure},
            _ShoppingListDeleteSchema,
        )
        return self._client._delete(
            endpoint=f"shopping-lists/{id}",
            params=params,
            response_schema_cls=ShoppingListSchema,
            force_delete=force_delete,
        )

    def delete_by_key(
        self,
        key: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        data_erasure: bool = None,
        force_delete: bool = False,
    ) -> ShoppingList:
        params = self._serialize_params(
            {"version": version, "expand": expand, "data_erasure": data_erasure},
            _ShoppingListDeleteSchema,
        )
        return self._client._delete(
            endpoint=f"shopping-lists/key={key}",
            params=params,
            response_schema_cls=ShoppingListSchema,
            force_delete=force_delete,
        )



from commercetools.testing.abstract import BaseBackend, BaseModel
from commercetools.testing.utils import create_commercetools_response


class InternalBackend(BaseBackend):
    path_prefix = r"/-/(?P<path>.*)"
    hostnames = ["auth.sphere.io", "localhost"]

    def urls(self):
        return [("^health$", "GET", self.health)]

    def health(self, request):
        response = create_commercetools_response(request, json={})
        return response



# DO NOT EDIT! This file is automatically generated
import datetime
import enum
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._common import CreatedBy, LastModifiedBy, LocalizedString
__all__ = [
    "AttributeBooleanType",
    "AttributeConstraintEnum",
    "AttributeConstraintEnumDraft",
    "AttributeDateTimeType",
    "AttributeDateType",
    "AttributeDefinition",
    "AttributeDefinitionDraft",
    "AttributeEnumType",
    "AttributeLocalizableTextType",
    "AttributeLocalizedEnumType",
    "AttributeLocalizedEnumValue",
    "AttributeMoneyType",
    "AttributeNestedType",
    "AttributeNumberType",
    "AttributePlainEnumValue",
    "AttributeReferenceType",
    "AttributeSetType",
    "AttributeTextType",
    "AttributeTimeType",
    "AttributeType",
    "ProductType",
    "ProductTypeAddAttributeDefinitionAction",
    "ProductTypeAddLocalizedEnumValueAction",
    "ProductTypeAddPlainEnumValueAction",
    "ProductTypeChangeAttributeConstraintAction",
    "ProductTypeChangeAttributeNameAction",
    "ProductTypeChangeAttributeOrderAction",
    "ProductTypeChangeAttributeOrderByNameAction",
    "ProductTypeChangeDescriptionAction",
    "ProductTypeChangeEnumKeyAction",
    "ProductTypeChangeInputHintAction",
    "ProductTypeChangeIsSearchableAction",
    "ProductTypeChangeLabelAction",
    "ProductTypeChangeLocalizedEnumValueLabelAction",
    "ProductTypeChangeLocalizedEnumValueOrderAction",
    "ProductTypeChangeNameAction",
    "ProductTypeChangePlainEnumValueLabelAction",
    "ProductTypeChangePlainEnumValueOrderAction",
    "ProductTypeDraft",
    "ProductTypePagedQueryResponse",
    "ProductTypeReference",
    "ProductTypeRemoveAttributeDefinitionAction",
    "ProductTypeRemoveEnumValuesAction",
    "ProductTypeResourceIdentifier",
    "ProductTypeSetInputTipAction",
    "ProductTypeSetKeyAction",
    "ProductTypeUpdate",
    "ProductTypeUpdateAction",
    "TextInputHint",
]


class AttributeConstraintEnum(enum.Enum):
    NONE = "None"
    UNIQUE = "Unique"
    COMBINATION_UNIQUE = "CombinationUnique"
    SAME_FOR_ALL = "SameForAll"


class AttributeConstraintEnumDraft(enum.Enum):
    NONE = "None"


class AttributeDefinition(_BaseType):
    #: :class:`commercetools.types.AttributeType`
    type: "AttributeType"
    #: :class:`str`
    name: str
    #: :class:`commercetools.types.LocalizedString`
    label: "LocalizedString"
    #: :class:`bool` `(Named` ``isRequired`` `in Commercetools)`
    is_required: bool
    #: :class:`commercetools.types.AttributeConstraintEnum` `(Named` ``attributeConstraint`` `in Commercetools)`
    attribute_constraint: "AttributeConstraintEnum"
    #: Optional :class:`commercetools.types.LocalizedString` `(Named` ``inputTip`` `in Commercetools)`
    input_tip: typing.Optional["LocalizedString"]
    #: :class:`commercetools.types.TextInputHint` `(Named` ``inputHint`` `in Commercetools)`
    input_hint: "TextInputHint"
    #: :class:`bool` `(Named` ``isSearchable`` `in Commercetools)`
    is_searchable: bool

    def __init__(
        self,
        *,
        type: "AttributeType",
        name: str,
        label: "LocalizedString",
        is_required: bool,
        attribute_constraint: "AttributeConstraintEnum",
        input_hint: "TextInputHint",
        is_searchable: bool,
        input_tip: typing.Optional["LocalizedString"] = None
    ) -> None:
        self.type = type
        self.name = name
        self.label = label
        self.is_required = is_required
        self.attribute_constraint = attribute_constraint
        self.input_tip = input_tip
        self.input_hint = input_hint
        self.is_searchable = is_searchable
        super().__init__()

    def __repr__(self) -> str:
        return (
            "AttributeDefinition(type=%r, name=%r, label=%r, is_required=%r, attribute_constraint=%r, input_tip=%r, input_hint=%r, is_searchable=%r)"
            % (
                self.type,
                self.name,
                self.label,
                self.is_required,
                self.attribute_constraint,
                self.input_tip,
                self.input_hint,
                self.is_searchable,
            )
        )


class AttributeDefinitionDraft(_BaseType):
    #: :class:`commercetools.types.AttributeType`
    type: "AttributeType"
    #: :class:`str`
    name: str
    #: :class:`commercetools.types.LocalizedString`
    label: "LocalizedString"
    #: :class:`bool` `(Named` ``isRequired`` `in Commercetools)`
    is_required: bool
    #: Optional :class:`commercetools.types.AttributeConstraintEnum` `(Named` ``attributeConstraint`` `in Commercetools)`
    attribute_constraint: typing.Optional["AttributeConstraintEnum"]
    #: Optional :class:`commercetools.types.LocalizedString` `(Named` ``inputTip`` `in Commercetools)`
    input_tip: typing.Optional["LocalizedString"]
    #: Optional :class:`commercetools.types.TextInputHint` `(Named` ``inputHint`` `in Commercetools)`
    input_hint: typing.Optional["TextInputHint"]
    #: Optional :class:`bool` `(Named` ``isSearchable`` `in Commercetools)`
    is_searchable: typing.Optional[bool]

    def __init__(
        self,
        *,
        type: "AttributeType",
        name: str,
        label: "LocalizedString",
        is_required: bool,
        attribute_constraint: typing.Optional["AttributeConstraintEnum"] = None,
        input_tip: typing.Optional["LocalizedString"] = None,
        input_hint: typing.Optional["TextInputHint"] = None,
        is_searchable: typing.Optional[bool] = None
    ) -> None:
        self.type = type
        self.name = name
        self.label = label
        self.is_required = is_required
        self.attribute_constraint = attribute_constraint
        self.input_tip = input_tip
        self.input_hint = input_hint
        self.is_searchable = is_searchable
        super().__init__()

    def __repr__(self) -> str:
        return (
            "AttributeDefinitionDraft(type=%r, name=%r, label=%r, is_required=%r, attribute_constraint=%r, input_tip=%r, input_hint=%r, is_searchable=%r)"
            % (
                self.type,
                self.name,
                self.label,
                self.is_required,
                self.attribute_constraint,
                self.input_tip,
                self.input_hint,
                self.is_searchable,
            )
        )


class AttributeLocalizedEnumValue(_BaseType):
    #: :class:`str`
    key: str
    #: :class:`commercetools.types.LocalizedString`
    label: "LocalizedString"

    def __init__(self, *, key: str, label: "LocalizedString") -> None:
        self.key = key
        self.label = label
        super().__init__()

    def __repr__(self) -> str:
        return "AttributeLocalizedEnumValue(key=%r, label=%r)" % (self.key, self.label)


class AttributePlainEnumValue(_BaseType):
    #: :class:`str`
    key: str
    #: :class:`str`
    label: str

    def __init__(self, *, key: str, label: str) -> None:
        self.key = key
        self.label = label
        super().__init__()

    def __repr__(self) -> str:
        return "AttributePlainEnumValue(key=%r, label=%r)" % (self.key, self.label)


class AttributeType(_BaseType):
    #: :class:`str`
    name: str

    def __init__(self, *, name: str) -> None:
        self.name = name
        super().__init__()

    def __repr__(self) -> str:
        return "AttributeType(name=%r)" % (self.name,)


class ProductType(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: :class:`str`
    name: str
    #: :class:`str`
    description: str
    #: Optional list of :class:`commercetools.types.AttributeDefinition`
    attributes: typing.Optional[typing.List["AttributeDefinition"]]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        name: str,
        description: str,
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        key: typing.Optional[str] = None,
        attributes: typing.Optional[typing.List["AttributeDefinition"]] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.key = key
        self.name = name
        self.description = description
        self.attributes = attributes
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "ProductType(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, key=%r, name=%r, description=%r, attributes=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.key,
                self.name,
                self.description,
                self.attributes,
            )
        )


class ProductTypeDraft(_BaseType):
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: :class:`str`
    name: str
    #: :class:`str`
    description: str
    #: Optional list of :class:`commercetools.types.AttributeDefinitionDraft`
    attributes: typing.Optional[typing.List["AttributeDefinitionDraft"]]

    def __init__(
        self,
        *,
        name: str,
        description: str,
        key: typing.Optional[str] = None,
        attributes: typing.Optional[typing.List["AttributeDefinitionDraft"]] = None
    ) -> None:
        self.key = key
        self.name = name
        self.description = description
        self.attributes = attributes
        super().__init__()

    def __repr__(self) -> str:
        return "ProductTypeDraft(key=%r, name=%r, description=%r, attributes=%r)" % (
            self.key,
            self.name,
            self.description,
            self.attributes,
        )


class ProductTypePagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.ProductType`
    results: typing.Sequence["ProductType"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["ProductType"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "ProductTypePagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class ProductTypeReference(Reference):
    #: Optional :class:`commercetools.types.ProductType`
    obj: typing.Optional["ProductType"]

    def __init__(self, *, id: str, obj: typing.Optional["ProductType"] = None) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.PRODUCT_TYPE, id=id)

    def __repr__(self) -> str:
        return "ProductTypeReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class ProductTypeResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.PRODUCT_TYPE, id=id, key=key)

    def __repr__(self) -> str:
        return "ProductTypeResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class ProductTypeUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "ProductTypeUpdate(version=%r, actions=%r)" % (
            self.version,
            self.actions,
        )


class ProductTypeUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "ProductTypeUpdateAction(action=%r)" % (self.action,)


class TextInputHint(enum.Enum):
    SINGLE_LINE = "SingleLine"
    MULTI_LINE = "MultiLine"


class AttributeBooleanType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="boolean")

    def __repr__(self) -> str:
        return "AttributeBooleanType(name=%r)" % (self.name,)


class AttributeDateTimeType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="datetime")

    def __repr__(self) -> str:
        return "AttributeDateTimeType(name=%r)" % (self.name,)


class AttributeDateType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="date")

    def __repr__(self) -> str:
        return "AttributeDateType(name=%r)" % (self.name,)


class AttributeEnumType(AttributeType):
    #: List of :class:`commercetools.types.AttributePlainEnumValue`
    values: typing.List["AttributePlainEnumValue"]

    def __init__(self, *, values: typing.List["AttributePlainEnumValue"]) -> None:
        self.values = values
        super().__init__(name="enum")

    def __repr__(self) -> str:
        return "AttributeEnumType(name=%r, values=%r)" % (self.name, self.values)


class AttributeLocalizableTextType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="ltext")

    def __repr__(self) -> str:
        return "AttributeLocalizableTextType(name=%r)" % (self.name,)


class AttributeLocalizedEnumType(AttributeType):
    #: List of :class:`commercetools.types.AttributeLocalizedEnumValue`
    values: typing.List["AttributeLocalizedEnumValue"]

    def __init__(self, *, values: typing.List["AttributeLocalizedEnumValue"]) -> None:
        self.values = values
        super().__init__(name="lenum")

    def __repr__(self) -> str:
        return "AttributeLocalizedEnumType(name=%r, values=%r)" % (
            self.name,
            self.values,
        )


class AttributeMoneyType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="money")

    def __repr__(self) -> str:
        return "AttributeMoneyType(name=%r)" % (self.name,)


class AttributeNestedType(AttributeType):
    #: :class:`commercetools.types.ProductTypeReference` `(Named` ``typeReference`` `in Commercetools)`
    type_reference: "ProductTypeReference"

    def __init__(self, *, type_reference: "ProductTypeReference") -> None:
        self.type_reference = type_reference
        super().__init__(name="nested")

    def __repr__(self) -> str:
        return "AttributeNestedType(name=%r, type_reference=%r)" % (
            self.name,
            self.type_reference,
        )


class AttributeNumberType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="number")

    def __repr__(self) -> str:
        return "AttributeNumberType(name=%r)" % (self.name,)


class AttributeReferenceType(AttributeType):
    #: :class:`commercetools.types.ReferenceTypeId` `(Named` ``referenceTypeId`` `in Commercetools)`
    reference_type_id: "ReferenceTypeId"

    def __init__(self, *, reference_type_id: "ReferenceTypeId") -> None:
        self.reference_type_id = reference_type_id
        super().__init__(name="reference")

    def __repr__(self) -> str:
        return "AttributeReferenceType(name=%r, reference_type_id=%r)" % (
            self.name,
            self.reference_type_id,
        )


class AttributeSetType(AttributeType):
    #: :class:`commercetools.types.AttributeType` `(Named` ``elementType`` `in Commercetools)`
    element_type: "AttributeType"

    def __init__(self, *, element_type: "AttributeType") -> None:
        self.element_type = element_type
        super().__init__(name="set")

    def __repr__(self) -> str:
        return "AttributeSetType(name=%r, element_type=%r)" % (
            self.name,
            self.element_type,
        )


class AttributeTextType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="text")

    def __repr__(self) -> str:
        return "AttributeTextType(name=%r)" % (self.name,)


class AttributeTimeType(AttributeType):
    def __init__(self) -> None:
        super().__init__(name="time")

    def __repr__(self) -> str:
        return "AttributeTimeType(name=%r)" % (self.name,)


class ProductTypeAddAttributeDefinitionAction(ProductTypeUpdateAction):
    #: :class:`commercetools.types.AttributeDefinitionDraft`
    attribute: "AttributeDefinitionDraft"

    def __init__(self, *, attribute: "AttributeDefinitionDraft") -> None:
        self.attribute = attribute
        super().__init__(action="addAttributeDefinition")

    def __repr__(self) -> str:
        return "ProductTypeAddAttributeDefinitionAction(action=%r, attribute=%r)" % (
            self.action,
            self.attribute,
        )


class ProductTypeAddLocalizedEnumValueAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`commercetools.types.AttributeLocalizedEnumValue`
    value: "AttributeLocalizedEnumValue"

    def __init__(
        self, *, attribute_name: str, value: "AttributeLocalizedEnumValue"
    ) -> None:
        self.attribute_name = attribute_name
        self.value = value
        super().__init__(action="addLocalizedEnumValue")

    def __repr__(self) -> str:
        return (
            "ProductTypeAddLocalizedEnumValueAction(action=%r, attribute_name=%r, value=%r)"
            % (self.action, self.attribute_name, self.value)
        )


class ProductTypeAddPlainEnumValueAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`commercetools.types.AttributePlainEnumValue`
    value: "AttributePlainEnumValue"

    def __init__(
        self, *, attribute_name: str, value: "AttributePlainEnumValue"
    ) -> None:
        self.attribute_name = attribute_name
        self.value = value
        super().__init__(action="addPlainEnumValue")

    def __repr__(self) -> str:
        return (
            "ProductTypeAddPlainEnumValueAction(action=%r, attribute_name=%r, value=%r)"
            % (self.action, self.attribute_name, self.value)
        )


class ProductTypeChangeAttributeConstraintAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`commercetools.types.AttributeConstraintEnumDraft` `(Named` ``newValue`` `in Commercetools)`
    new_value: "AttributeConstraintEnumDraft"

    def __init__(
        self, *, attribute_name: str, new_value: "AttributeConstraintEnumDraft"
    ) -> None:
        self.attribute_name = attribute_name
        self.new_value = new_value
        super().__init__(action="changeAttributeConstraint")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeAttributeConstraintAction(action=%r, attribute_name=%r, new_value=%r)"
            % (self.action, self.attribute_name, self.new_value)
        )


class ProductTypeChangeAttributeNameAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`str` `(Named` ``newAttributeName`` `in Commercetools)`
    new_attribute_name: str

    def __init__(self, *, attribute_name: str, new_attribute_name: str) -> None:
        self.attribute_name = attribute_name
        self.new_attribute_name = new_attribute_name
        super().__init__(action="changeAttributeName")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeAttributeNameAction(action=%r, attribute_name=%r, new_attribute_name=%r)"
            % (self.action, self.attribute_name, self.new_attribute_name)
        )


class ProductTypeChangeAttributeOrderAction(ProductTypeUpdateAction):
    #: List of :class:`commercetools.types.AttributeDefinition`
    attributes: typing.List["AttributeDefinition"]

    def __init__(self, *, attributes: typing.List["AttributeDefinition"]) -> None:
        self.attributes = attributes
        super().__init__(action="changeAttributeOrder")

    def __repr__(self) -> str:
        return "ProductTypeChangeAttributeOrderAction(action=%r, attributes=%r)" % (
            self.action,
            self.attributes,
        )


class ProductTypeChangeAttributeOrderByNameAction(ProductTypeUpdateAction):
    #: List of :class:`str` `(Named` ``attributeNames`` `in Commercetools)`
    attribute_names: typing.List[str]

    def __init__(self, *, attribute_names: typing.List[str]) -> None:
        self.attribute_names = attribute_names
        super().__init__(action="changeAttributeOrderByName")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeAttributeOrderByNameAction(action=%r, attribute_names=%r)"
            % (self.action, self.attribute_names)
        )


class ProductTypeChangeDescriptionAction(ProductTypeUpdateAction):
    #: :class:`str`
    description: str

    def __init__(self, *, description: str) -> None:
        self.description = description
        super().__init__(action="changeDescription")

    def __repr__(self) -> str:
        return "ProductTypeChangeDescriptionAction(action=%r, description=%r)" % (
            self.action,
            self.description,
        )


class ProductTypeChangeEnumKeyAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`str`
    key: str
    #: :class:`str` `(Named` ``newKey`` `in Commercetools)`
    new_key: str

    def __init__(self, *, attribute_name: str, key: str, new_key: str) -> None:
        self.attribute_name = attribute_name
        self.key = key
        self.new_key = new_key
        super().__init__(action="changeEnumKey")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeEnumKeyAction(action=%r, attribute_name=%r, key=%r, new_key=%r)"
            % (self.action, self.attribute_name, self.key, self.new_key)
        )


class ProductTypeChangeInputHintAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`commercetools.types.TextInputHint` `(Named` ``newValue`` `in Commercetools)`
    new_value: "TextInputHint"

    def __init__(self, *, attribute_name: str, new_value: "TextInputHint") -> None:
        self.attribute_name = attribute_name
        self.new_value = new_value
        super().__init__(action="changeInputHint")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeInputHintAction(action=%r, attribute_name=%r, new_value=%r)"
            % (self.action, self.attribute_name, self.new_value)
        )


class ProductTypeChangeIsSearchableAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`bool` `(Named` ``isSearchable`` `in Commercetools)`
    is_searchable: bool

    def __init__(self, *, attribute_name: str, is_searchable: bool) -> None:
        self.attribute_name = attribute_name
        self.is_searchable = is_searchable
        super().__init__(action="changeIsSearchable")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeIsSearchableAction(action=%r, attribute_name=%r, is_searchable=%r)"
            % (self.action, self.attribute_name, self.is_searchable)
        )


class ProductTypeChangeLabelAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`commercetools.types.LocalizedString`
    label: "LocalizedString"

    def __init__(self, *, attribute_name: str, label: "LocalizedString") -> None:
        self.attribute_name = attribute_name
        self.label = label
        super().__init__(action="changeLabel")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeLabelAction(action=%r, attribute_name=%r, label=%r)"
            % (self.action, self.attribute_name, self.label)
        )


class ProductTypeChangeLocalizedEnumValueLabelAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`commercetools.types.AttributeLocalizedEnumValue` `(Named` ``newValue`` `in Commercetools)`
    new_value: "AttributeLocalizedEnumValue"

    def __init__(
        self, *, attribute_name: str, new_value: "AttributeLocalizedEnumValue"
    ) -> None:
        self.attribute_name = attribute_name
        self.new_value = new_value
        super().__init__(action="changeLocalizedEnumValueLabel")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeLocalizedEnumValueLabelAction(action=%r, attribute_name=%r, new_value=%r)"
            % (self.action, self.attribute_name, self.new_value)
        )


class ProductTypeChangeLocalizedEnumValueOrderAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: List of :class:`commercetools.types.AttributeLocalizedEnumValue`
    values: typing.List["AttributeLocalizedEnumValue"]

    def __init__(
        self, *, attribute_name: str, values: typing.List["AttributeLocalizedEnumValue"]
    ) -> None:
        self.attribute_name = attribute_name
        self.values = values
        super().__init__(action="changeLocalizedEnumValueOrder")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangeLocalizedEnumValueOrderAction(action=%r, attribute_name=%r, values=%r)"
            % (self.action, self.attribute_name, self.values)
        )


class ProductTypeChangeNameAction(ProductTypeUpdateAction):
    #: :class:`str`
    name: str

    def __init__(self, *, name: str) -> None:
        self.name = name
        super().__init__(action="changeName")

    def __repr__(self) -> str:
        return "ProductTypeChangeNameAction(action=%r, name=%r)" % (
            self.action,
            self.name,
        )


class ProductTypeChangePlainEnumValueLabelAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: :class:`commercetools.types.AttributePlainEnumValue` `(Named` ``newValue`` `in Commercetools)`
    new_value: "AttributePlainEnumValue"

    def __init__(
        self, *, attribute_name: str, new_value: "AttributePlainEnumValue"
    ) -> None:
        self.attribute_name = attribute_name
        self.new_value = new_value
        super().__init__(action="changePlainEnumValueLabel")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangePlainEnumValueLabelAction(action=%r, attribute_name=%r, new_value=%r)"
            % (self.action, self.attribute_name, self.new_value)
        )


class ProductTypeChangePlainEnumValueOrderAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: List of :class:`commercetools.types.AttributePlainEnumValue`
    values: typing.List["AttributePlainEnumValue"]

    def __init__(
        self, *, attribute_name: str, values: typing.List["AttributePlainEnumValue"]
    ) -> None:
        self.attribute_name = attribute_name
        self.values = values
        super().__init__(action="changePlainEnumValueOrder")

    def __repr__(self) -> str:
        return (
            "ProductTypeChangePlainEnumValueOrderAction(action=%r, attribute_name=%r, values=%r)"
            % (self.action, self.attribute_name, self.values)
        )


class ProductTypeRemoveAttributeDefinitionAction(ProductTypeUpdateAction):
    #: :class:`str`
    name: str

    def __init__(self, *, name: str) -> None:
        self.name = name
        super().__init__(action="removeAttributeDefinition")

    def __repr__(self) -> str:
        return "ProductTypeRemoveAttributeDefinitionAction(action=%r, name=%r)" % (
            self.action,
            self.name,
        )


class ProductTypeRemoveEnumValuesAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: List of :class:`str`
    keys: typing.List[str]

    def __init__(self, *, attribute_name: str, keys: typing.List[str]) -> None:
        self.attribute_name = attribute_name
        self.keys = keys
        super().__init__(action="removeEnumValues")

    def __repr__(self) -> str:
        return (
            "ProductTypeRemoveEnumValuesAction(action=%r, attribute_name=%r, keys=%r)"
            % (self.action, self.attribute_name, self.keys)
        )


class ProductTypeSetInputTipAction(ProductTypeUpdateAction):
    #: :class:`str` `(Named` ``attributeName`` `in Commercetools)`
    attribute_name: str
    #: Optional :class:`commercetools.types.LocalizedString` `(Named` ``inputTip`` `in Commercetools)`
    input_tip: typing.Optional["LocalizedString"]

    def __init__(
        self,
        *,
        attribute_name: str,
        input_tip: typing.Optional["LocalizedString"] = None
    ) -> None:
        self.attribute_name = attribute_name
        self.input_tip = input_tip
        super().__init__(action="setInputTip")

    def __repr__(self) -> str:
        return (
            "ProductTypeSetInputTipAction(action=%r, attribute_name=%r, input_tip=%r)"
            % (self.action, self.attribute_name, self.input_tip)
        )


class ProductTypeSetKeyAction(ProductTypeUpdateAction):
    #: Optional :class:`str`
    key: typing.Optional[str]

    def __init__(self, *, key: typing.Optional[str] = None) -> None:
        self.key = key
        super().__init__(action="setKey")

    def __repr__(self) -> str:
        return "ProductTypeSetKeyAction(action=%r, key=%r)" % (self.action, self.key)



# DO NOT EDIT! This file is automatically generated
import datetime
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._common import CreatedBy, LastModifiedBy
    from ._type import CustomFields, FieldContainer, TypeResourceIdentifier
__all__ = [
    "CustomerGroup",
    "CustomerGroupChangeNameAction",
    "CustomerGroupDraft",
    "CustomerGroupPagedQueryResponse",
    "CustomerGroupReference",
    "CustomerGroupResourceIdentifier",
    "CustomerGroupSetCustomFieldAction",
    "CustomerGroupSetCustomTypeAction",
    "CustomerGroupSetKeyAction",
    "CustomerGroupUpdate",
    "CustomerGroupUpdateAction",
]


class CustomerGroup(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: :class:`str`
    name: str
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        name: str,
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        key: typing.Optional[str] = None,
        custom: typing.Optional["CustomFields"] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.key = key
        self.name = name
        self.custom = custom
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "CustomerGroup(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, key=%r, name=%r, custom=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.key,
                self.name,
                self.custom,
            )
        )


class CustomerGroupDraft(_BaseType):
    #: Optional :class:`str`
    key: typing.Optional[str]
    #: :class:`str` `(Named` ``groupName`` `in Commercetools)`
    group_name: str
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]

    def __init__(
        self,
        *,
        group_name: str,
        key: typing.Optional[str] = None,
        custom: typing.Optional["CustomFields"] = None
    ) -> None:
        self.key = key
        self.group_name = group_name
        self.custom = custom
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerGroupDraft(key=%r, group_name=%r, custom=%r)" % (
            self.key,
            self.group_name,
            self.custom,
        )


class CustomerGroupPagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.CustomerGroup`
    results: typing.Sequence["CustomerGroup"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["CustomerGroup"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "CustomerGroupPagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class CustomerGroupReference(Reference):
    #: Optional :class:`commercetools.types.CustomerGroup`
    obj: typing.Optional["CustomerGroup"]

    def __init__(
        self, *, id: str, obj: typing.Optional["CustomerGroup"] = None
    ) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.CUSTOMER_GROUP, id=id)

    def __repr__(self) -> str:
        return "CustomerGroupReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class CustomerGroupResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.CUSTOMER_GROUP, id=id, key=key)

    def __repr__(self) -> str:
        return "CustomerGroupResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class CustomerGroupUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerGroupUpdate(version=%r, actions=%r)" % (
            self.version,
            self.actions,
        )


class CustomerGroupUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "CustomerGroupUpdateAction(action=%r)" % (self.action,)


class CustomerGroupChangeNameAction(CustomerGroupUpdateAction):
    #: :class:`str`
    name: str

    def __init__(self, *, name: str) -> None:
        self.name = name
        super().__init__(action="changeName")

    def __repr__(self) -> str:
        return "CustomerGroupChangeNameAction(action=%r, name=%r)" % (
            self.action,
            self.name,
        )


class CustomerGroupSetCustomFieldAction(CustomerGroupUpdateAction):
    #: :class:`str`
    name: str
    #: Optional :class:`typing.Any`
    value: typing.Optional[typing.Any]

    def __init__(self, *, name: str, value: typing.Optional[typing.Any] = None) -> None:
        self.name = name
        self.value = value
        super().__init__(action="setCustomField")

    def __repr__(self) -> str:
        return "CustomerGroupSetCustomFieldAction(action=%r, name=%r, value=%r)" % (
            self.action,
            self.name,
            self.value,
        )


class CustomerGroupSetCustomTypeAction(CustomerGroupUpdateAction):
    #: Optional :class:`commercetools.types.TypeResourceIdentifier`
    type: typing.Optional["TypeResourceIdentifier"]
    #: Optional :class:`commercetools.types.FieldContainer`
    fields: typing.Optional["FieldContainer"]

    def __init__(
        self,
        *,
        type: typing.Optional["TypeResourceIdentifier"] = None,
        fields: typing.Optional["FieldContainer"] = None
    ) -> None:
        self.type = type
        self.fields = fields
        super().__init__(action="setCustomType")

    def __repr__(self) -> str:
        return "CustomerGroupSetCustomTypeAction(action=%r, type=%r, fields=%r)" % (
            self.action,
            self.type,
            self.fields,
        )


class CustomerGroupSetKeyAction(CustomerGroupUpdateAction):
    #: Optional :class:`str`
    key: typing.Optional[str]

    def __init__(self, *, key: typing.Optional[str] = None) -> None:
        self.key = key
        super().__init__(action="setKey")

    def __repr__(self) -> str:
        return "CustomerGroupSetKeyAction(action=%r, key=%r)" % (self.action, self.key)



# DO NOT EDIT! This file is automatically generated
import marshmallow
import marshmallow_enum

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)

__all__ = [
    "CartClassificationTierSchema",
    "CartScoreTierSchema",
    "CartValueTierSchema",
    "PriceFunctionSchema",
    "ShippingMethodAddShippingRateActionSchema",
    "ShippingMethodAddZoneActionSchema",
    "ShippingMethodChangeIsDefaultActionSchema",
    "ShippingMethodChangeNameActionSchema",
    "ShippingMethodChangeTaxCategoryActionSchema",
    "ShippingMethodDraftSchema",
    "ShippingMethodPagedQueryResponseSchema",
    "ShippingMethodReferenceSchema",
    "ShippingMethodRemoveShippingRateActionSchema",
    "ShippingMethodRemoveZoneActionSchema",
    "ShippingMethodResourceIdentifierSchema",
    "ShippingMethodSchema",
    "ShippingMethodSetDescriptionActionSchema",
    "ShippingMethodSetKeyActionSchema",
    "ShippingMethodSetLocalizedDescriptionActionSchema",
    "ShippingMethodSetPredicateActionSchema",
    "ShippingMethodUpdateActionSchema",
    "ShippingMethodUpdateSchema",
    "ShippingRateDraftSchema",
    "ShippingRatePriceTierSchema",
    "ShippingRateSchema",
    "ZoneRateDraftSchema",
    "ZoneRateSchema",
]


class PriceFunctionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.PriceFunction`."""

    currency_code = marshmallow.fields.String(data_key="currencyCode")
    function = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.PriceFunction(**data)


class ShippingMethodDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodDraft`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)
    name = marshmallow.fields.String(allow_none=True)
    description = marshmallow.fields.String(allow_none=True, missing=None)
    localized_description = LocalizedStringField(
        allow_none=True, missing=None, data_key="localizedDescription"
    )
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="taxCategory",
    )
    zone_rates = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ZoneRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="zoneRates",
    )
    is_default = marshmallow.fields.Bool(allow_none=True, data_key="isDefault")
    predicate = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShippingMethodDraft(**data)


class ShippingMethodPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True, missing=None)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True, missing=None)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingMethodSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShippingMethodPagedQueryResponse(**data)


class ShippingMethodReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingMethodSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ShippingMethodReference(**data)


class ShippingMethodResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ShippingMethodResourceIdentifier(**data)


class ShippingMethodSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethod`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)
    name = marshmallow.fields.String(allow_none=True)
    description = marshmallow.fields.String(allow_none=True, missing=None)
    localized_description = LocalizedStringField(
        allow_none=True, missing=None, data_key="localizedDescription"
    )
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="taxCategory",
    )
    zone_rates = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ZoneRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="zoneRates",
    )
    is_default = marshmallow.fields.Bool(allow_none=True, data_key="isDefault")
    predicate = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShippingMethod(**data)


class ShippingMethodUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodUpdateAction(**data)


class ShippingMethodUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addShippingRate": "commercetools._schemas._shipping_method.ShippingMethodAddShippingRateActionSchema",
                "addZone": "commercetools._schemas._shipping_method.ShippingMethodAddZoneActionSchema",
                "changeIsDefault": "commercetools._schemas._shipping_method.ShippingMethodChangeIsDefaultActionSchema",
                "changeName": "commercetools._schemas._shipping_method.ShippingMethodChangeNameActionSchema",
                "changeTaxCategory": "commercetools._schemas._shipping_method.ShippingMethodChangeTaxCategoryActionSchema",
                "removeShippingRate": "commercetools._schemas._shipping_method.ShippingMethodRemoveShippingRateActionSchema",
                "removeZone": "commercetools._schemas._shipping_method.ShippingMethodRemoveZoneActionSchema",
                "setDescription": "commercetools._schemas._shipping_method.ShippingMethodSetDescriptionActionSchema",
                "setKey": "commercetools._schemas._shipping_method.ShippingMethodSetKeyActionSchema",
                "setLocalizedDescription": "commercetools._schemas._shipping_method.ShippingMethodSetLocalizedDescriptionActionSchema",
                "setPredicate": "commercetools._schemas._shipping_method.ShippingMethodSetPredicateActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShippingMethodUpdate(**data)


class ShippingRateDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingRateDraft`."""

    price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    free_above = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="freeAbove",
    )
    tiers = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("type", "type"),
            discriminator_schemas={
                "CartClassification": "commercetools._schemas._shipping_method.CartClassificationTierSchema",
                "CartScore": "commercetools._schemas._shipping_method.CartScoreTierSchema",
                "CartValue": "commercetools._schemas._shipping_method.CartValueTierSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShippingRateDraft(**data)


class ShippingRatePriceTierSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingRatePriceTier`."""

    type = marshmallow_enum.EnumField(types.ShippingRateTierType, by_value=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ShippingRatePriceTier(**data)


class ShippingRateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ShippingRate`."""

    price = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    free_above = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="freeAbove",
    )
    is_matching = marshmallow.fields.Bool(
        allow_none=True, missing=None, data_key="isMatching"
    )
    tiers = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("type", "type"),
            discriminator_schemas={
                "CartClassification": "commercetools._schemas._shipping_method.CartClassificationTierSchema",
                "CartScore": "commercetools._schemas._shipping_method.CartScoreTierSchema",
                "CartValue": "commercetools._schemas._shipping_method.CartValueTierSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        )
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ShippingRate(**data)


class ZoneRateDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ZoneRateDraft`."""

    zone = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    shipping_rates = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="shippingRates",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ZoneRateDraft(**data)


class ZoneRateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ZoneRate`."""

    zone = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    shipping_rates = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        data_key="shippingRates",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ZoneRate(**data)


class CartClassificationTierSchema(ShippingRatePriceTierSchema):
    """Marshmallow schema for :class:`commercetools.types.CartClassificationTier`."""

    value = marshmallow.fields.String(allow_none=True)
    price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    is_matching = marshmallow.fields.Bool(
        allow_none=True, missing=None, data_key="isMatching"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartClassificationTier(**data)


class CartScoreTierSchema(ShippingRatePriceTierSchema):
    """Marshmallow schema for :class:`commercetools.types.CartScoreTier`."""

    score = marshmallow.fields.Integer(allow_none=True)
    price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    price_function = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.PriceFunctionSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="priceFunction",
    )
    is_matching = marshmallow.fields.Bool(
        allow_none=True, missing=None, data_key="isMatching"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartScoreTier(**data)


class CartValueTierSchema(ShippingRatePriceTierSchema):
    """Marshmallow schema for :class:`commercetools.types.CartValueTier`."""

    minimum_cent_amount = marshmallow.fields.Integer(
        allow_none=True, data_key="minimumCentAmount"
    )
    price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    is_matching = marshmallow.fields.Bool(
        allow_none=True, missing=None, data_key="isMatching"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.CartValueTier(**data)


class ShippingMethodAddShippingRateActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodAddShippingRateAction`."""

    zone = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    shipping_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shippingRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodAddShippingRateAction(**data)


class ShippingMethodAddZoneActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodAddZoneAction`."""

    zone = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodAddZoneAction(**data)


class ShippingMethodChangeIsDefaultActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodChangeIsDefaultAction`."""

    is_default = marshmallow.fields.Bool(allow_none=True, data_key="isDefault")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodChangeIsDefaultAction(**data)


class ShippingMethodChangeNameActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodChangeNameAction`."""

    name = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodChangeNameAction(**data)


class ShippingMethodChangeTaxCategoryActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodChangeTaxCategoryAction`."""

    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="taxCategory",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodChangeTaxCategoryAction(**data)


class ShippingMethodRemoveShippingRateActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodRemoveShippingRateAction`."""

    zone = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    shipping_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shippingRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodRemoveShippingRateAction(**data)


class ShippingMethodRemoveZoneActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodRemoveZoneAction`."""

    zone = helpers.LazyNestedField(
        nested="commercetools._schemas._zone.ZoneResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodRemoveZoneAction(**data)


class ShippingMethodSetDescriptionActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodSetDescriptionAction`."""

    description = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodSetDescriptionAction(**data)


class ShippingMethodSetKeyActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodSetKeyAction(**data)


class ShippingMethodSetLocalizedDescriptionActionSchema(
    ShippingMethodUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodSetLocalizedDescriptionAction`."""

    localized_description = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="localizedDescription"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodSetLocalizedDescriptionAction(**data)


class ShippingMethodSetPredicateActionSchema(ShippingMethodUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ShippingMethodSetPredicateAction`."""

    predicate = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ShippingMethodSetPredicateAction(**data)



# DO NOT EDIT! This file is automatically generated
import typing

from commercetools._schemas._product_type import (
    ProductTypeDraftSchema,
    ProductTypePagedQueryResponseSchema,
    ProductTypeSchema,
    ProductTypeUpdateSchema,
)
from commercetools.helpers import RemoveEmptyValuesMixin
from commercetools.types._product_type import (
    ProductType,
    ProductTypeDraft,
    ProductTypePagedQueryResponse,
    ProductTypeUpdate,
    ProductTypeUpdateAction,
)
from commercetools.typing import OptionalListStr

from . import abstract, traits


class _ProductTypeQuerySchema(
    traits.ExpandableSchema,
    traits.SortableSchema,
    traits.PagingSchema,
    traits.QuerySchema,
):
    pass


class _ProductTypeUpdateSchema(traits.ExpandableSchema, traits.VersionedSchema):
    pass


class _ProductTypeDeleteSchema(traits.VersionedSchema, traits.ExpandableSchema):
    pass


class ProductTypeService(abstract.AbstractService):
    """Product Types are used to describe common characteristics, most importantly
    common custom attributes,

    of many concrete products.
    """

    def get_by_id(self, id: str, *, expand: OptionalListStr = None) -> ProductType:
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"product-types/{id}", params=params, schema_cls=ProductTypeSchema
        )

    def get_by_key(self, key: str, *, expand: OptionalListStr = None) -> ProductType:
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._get(
            endpoint=f"product-types/key={key}",
            params=params,
            schema_cls=ProductTypeSchema,
        )

    def query(
        self,
        *,
        expand: OptionalListStr = None,
        sort: OptionalListStr = None,
        limit: int = None,
        offset: int = None,
        with_total: bool = None,
        where: OptionalListStr = None,
        predicate_var: typing.Dict[str, str] = None,
    ) -> ProductTypePagedQueryResponse:
        """Product Types are used to describe common characteristics, most
        importantly common custom attributes, of many concrete products.
        """
        params = self._serialize_params(
            {
                "expand": expand,
                "sort": sort,
                "limit": limit,
                "offset": offset,
                "with_total": with_total,
                "where": where,
                "predicate_var": predicate_var,
            },
            _ProductTypeQuerySchema,
        )
        return self._client._get(
            endpoint="product-types",
            params=params,
            schema_cls=ProductTypePagedQueryResponseSchema,
        )

    def create(
        self, draft: ProductTypeDraft, *, expand: OptionalListStr = None
    ) -> ProductType:
        """Product Types are used to describe common characteristics, most
        importantly common custom attributes, of many concrete products.
        """
        params = self._serialize_params({"expand": expand}, traits.ExpandableSchema)
        return self._client._post(
            endpoint="product-types",
            params=params,
            data_object=draft,
            request_schema_cls=ProductTypeDraftSchema,
            response_schema_cls=ProductTypeSchema,
        )

    def update_by_id(
        self,
        id: str,
        version: int,
        actions: typing.List[ProductTypeUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> ProductType:
        params = self._serialize_params({"expand": expand}, _ProductTypeUpdateSchema)
        update_action = ProductTypeUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"product-types/{id}",
            params=params,
            data_object=update_action,
            request_schema_cls=ProductTypeUpdateSchema,
            response_schema_cls=ProductTypeSchema,
            force_update=force_update,
        )

    def update_by_key(
        self,
        key: str,
        version: int,
        actions: typing.List[ProductTypeUpdateAction],
        *,
        expand: OptionalListStr = None,
        force_update: bool = False,
    ) -> ProductType:
        params = self._serialize_params({"expand": expand}, _ProductTypeUpdateSchema)
        update_action = ProductTypeUpdate(version=version, actions=actions)
        return self._client._post(
            endpoint=f"product-types/key={key}",
            params=params,
            data_object=update_action,
            request_schema_cls=ProductTypeUpdateSchema,
            response_schema_cls=ProductTypeSchema,
            force_update=force_update,
        )

    def delete_by_id(
        self,
        id: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> ProductType:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _ProductTypeDeleteSchema
        )
        return self._client._delete(
            endpoint=f"product-types/{id}",
            params=params,
            response_schema_cls=ProductTypeSchema,
            force_delete=force_delete,
        )

    def delete_by_key(
        self,
        key: str,
        version: int,
        *,
        expand: OptionalListStr = None,
        force_delete: bool = False,
    ) -> ProductType:
        params = self._serialize_params(
            {"version": version, "expand": expand}, _ProductTypeDeleteSchema
        )
        return self._client._delete(
            endpoint=f"product-types/key={key}",
            params=params,
            response_schema_cls=ProductTypeSchema,
            force_delete=force_delete,
        )



# DO NOT EDIT! This file is automatically generated
import typing

from commercetools._schemas._customer import CustomerSignInResultSchema
from commercetools.helpers import RemoveEmptyValuesMixin
from commercetools.types._customer import CustomerSignInResult
from commercetools.typing import OptionalListStr

from . import abstract, traits


class LoginService(abstract.AbstractService):
    """Retrieves the authenticated customer."""

    def create(self) -> CustomerSignInResult:
        """Authenticate Customer (Sign In).

        Retrieves the authenticated customer (a customer that matches the given
        email/password pair). If used with an access token for Anonymous
        Sessions, all orders and carts belonging to the anonymousId will be
        assigned to the newly created customer. If a cart is is returned as part
        of the CustomerSignInResult, it has been recalculated (It will have up-
        to-date prices, taxes and discounts, and invalid line items have been
        removed.).   Retrieves the authenticated customer.
        """
        params: typing.Dict[str, str] = {}
        return self._client._post(
            endpoint="login",
            params=params,
            data_object=None,
            request_schema_cls=None,
            response_schema_cls=CustomerSignInResultSchema,
        )



# DO NOT EDIT! This file is automatically generated
import datetime
import typing

from commercetools.types._abstract import _BaseType
from commercetools.types._common import (
    BaseResource,
    Reference,
    ReferenceTypeId,
    ResourceIdentifier,
)

if typing.TYPE_CHECKING:
    from ._cart_discount import CartDiscountReference, CartDiscountResourceIdentifier
    from ._common import CreatedBy, LastModifiedBy, LocalizedString
    from ._type import (
        CustomFields,
        CustomFieldsDraft,
        FieldContainer,
        TypeResourceIdentifier,
    )
__all__ = [
    "DiscountCode",
    "DiscountCodeChangeCartDiscountsAction",
    "DiscountCodeChangeGroupsAction",
    "DiscountCodeChangeIsActiveAction",
    "DiscountCodeDraft",
    "DiscountCodePagedQueryResponse",
    "DiscountCodeReference",
    "DiscountCodeResourceIdentifier",
    "DiscountCodeSetCartPredicateAction",
    "DiscountCodeSetCustomFieldAction",
    "DiscountCodeSetCustomTypeAction",
    "DiscountCodeSetDescriptionAction",
    "DiscountCodeSetMaxApplicationsAction",
    "DiscountCodeSetMaxApplicationsPerCustomerAction",
    "DiscountCodeSetNameAction",
    "DiscountCodeSetValidFromAction",
    "DiscountCodeSetValidFromAndUntilAction",
    "DiscountCodeSetValidUntilAction",
    "DiscountCodeUpdate",
    "DiscountCodeUpdateAction",
]


class DiscountCode(BaseResource):
    #: :class:`str`
    id: str
    #: :class:`int`
    version: int
    #: :class:`datetime.datetime` `(Named` ``createdAt`` `in Commercetools)`
    created_at: datetime.datetime
    #: :class:`datetime.datetime` `(Named` ``lastModifiedAt`` `in Commercetools)`
    last_modified_at: datetime.datetime
    #: Optional :class:`commercetools.types.LastModifiedBy` `(Named` ``lastModifiedBy`` `in Commercetools)`
    last_modified_by: typing.Optional["LastModifiedBy"]
    #: Optional :class:`commercetools.types.CreatedBy` `(Named` ``createdBy`` `in Commercetools)`
    created_by: typing.Optional["CreatedBy"]
    #: Optional :class:`commercetools.types.LocalizedString`
    name: typing.Optional["LocalizedString"]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: :class:`str`
    code: str
    #: List of :class:`commercetools.types.CartDiscountReference` `(Named` ``cartDiscounts`` `in Commercetools)`
    cart_discounts: typing.List["CartDiscountReference"]
    #: Optional :class:`str` `(Named` ``cartPredicate`` `in Commercetools)`
    cart_predicate: typing.Optional[str]
    #: :class:`bool` `(Named` ``isActive`` `in Commercetools)`
    is_active: bool
    #: List of :class:`commercetools.types.Reference`
    references: typing.List["Reference"]
    #: Optional :class:`int` `(Named` ``maxApplications`` `in Commercetools)`
    max_applications: typing.Optional[int]
    #: Optional :class:`int` `(Named` ``maxApplicationsPerCustomer`` `in Commercetools)`
    max_applications_per_customer: typing.Optional[int]
    #: Optional :class:`commercetools.types.CustomFields`
    custom: typing.Optional["CustomFields"]
    #: List of :class:`str`
    groups: typing.List[str]
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self,
        *,
        id: str,
        version: int,
        created_at: datetime.datetime,
        last_modified_at: datetime.datetime,
        code: str,
        cart_discounts: typing.List["CartDiscountReference"],
        is_active: bool,
        references: typing.List["Reference"],
        groups: typing.List[str],
        last_modified_by: typing.Optional["LastModifiedBy"] = None,
        created_by: typing.Optional["CreatedBy"] = None,
        name: typing.Optional["LocalizedString"] = None,
        description: typing.Optional["LocalizedString"] = None,
        cart_predicate: typing.Optional[str] = None,
        max_applications: typing.Optional[int] = None,
        max_applications_per_customer: typing.Optional[int] = None,
        custom: typing.Optional["CustomFields"] = None,
        valid_from: typing.Optional[datetime.datetime] = None,
        valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.id = id
        self.version = version
        self.created_at = created_at
        self.last_modified_at = last_modified_at
        self.last_modified_by = last_modified_by
        self.created_by = created_by
        self.name = name
        self.description = description
        self.code = code
        self.cart_discounts = cart_discounts
        self.cart_predicate = cart_predicate
        self.is_active = is_active
        self.references = references
        self.max_applications = max_applications
        self.max_applications_per_customer = max_applications_per_customer
        self.custom = custom
        self.groups = groups
        self.valid_from = valid_from
        self.valid_until = valid_until
        super().__init__(
            id=id,
            version=version,
            created_at=created_at,
            last_modified_at=last_modified_at,
        )

    def __repr__(self) -> str:
        return (
            "DiscountCode(id=%r, version=%r, created_at=%r, last_modified_at=%r, last_modified_by=%r, created_by=%r, name=%r, description=%r, code=%r, cart_discounts=%r, cart_predicate=%r, is_active=%r, references=%r, max_applications=%r, max_applications_per_customer=%r, custom=%r, groups=%r, valid_from=%r, valid_until=%r)"
            % (
                self.id,
                self.version,
                self.created_at,
                self.last_modified_at,
                self.last_modified_by,
                self.created_by,
                self.name,
                self.description,
                self.code,
                self.cart_discounts,
                self.cart_predicate,
                self.is_active,
                self.references,
                self.max_applications,
                self.max_applications_per_customer,
                self.custom,
                self.groups,
                self.valid_from,
                self.valid_until,
            )
        )


class DiscountCodeDraft(_BaseType):
    #: Optional :class:`commercetools.types.LocalizedString`
    name: typing.Optional["LocalizedString"]
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]
    #: :class:`str`
    code: str
    #: List of :class:`commercetools.types.CartDiscountResourceIdentifier` `(Named` ``cartDiscounts`` `in Commercetools)`
    cart_discounts: typing.List["CartDiscountResourceIdentifier"]
    #: Optional :class:`str` `(Named` ``cartPredicate`` `in Commercetools)`
    cart_predicate: typing.Optional[str]
    #: Optional :class:`bool` `(Named` ``isActive`` `in Commercetools)`
    is_active: typing.Optional[bool]
    #: Optional :class:`int` `(Named` ``maxApplications`` `in Commercetools)`
    max_applications: typing.Optional[int]
    #: Optional :class:`int` `(Named` ``maxApplicationsPerCustomer`` `in Commercetools)`
    max_applications_per_customer: typing.Optional[int]
    #: Optional :class:`commercetools.types.CustomFieldsDraft`
    custom: typing.Optional["CustomFieldsDraft"]
    #: Optional :class:`list`
    groups: typing.Optional[list]
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self,
        *,
        code: str,
        cart_discounts: typing.List["CartDiscountResourceIdentifier"],
        name: typing.Optional["LocalizedString"] = None,
        description: typing.Optional["LocalizedString"] = None,
        cart_predicate: typing.Optional[str] = None,
        is_active: typing.Optional[bool] = None,
        max_applications: typing.Optional[int] = None,
        max_applications_per_customer: typing.Optional[int] = None,
        custom: typing.Optional["CustomFieldsDraft"] = None,
        groups: typing.Optional[list] = None,
        valid_from: typing.Optional[datetime.datetime] = None,
        valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.name = name
        self.description = description
        self.code = code
        self.cart_discounts = cart_discounts
        self.cart_predicate = cart_predicate
        self.is_active = is_active
        self.max_applications = max_applications
        self.max_applications_per_customer = max_applications_per_customer
        self.custom = custom
        self.groups = groups
        self.valid_from = valid_from
        self.valid_until = valid_until
        super().__init__()

    def __repr__(self) -> str:
        return (
            "DiscountCodeDraft(name=%r, description=%r, code=%r, cart_discounts=%r, cart_predicate=%r, is_active=%r, max_applications=%r, max_applications_per_customer=%r, custom=%r, groups=%r, valid_from=%r, valid_until=%r)"
            % (
                self.name,
                self.description,
                self.code,
                self.cart_discounts,
                self.cart_predicate,
                self.is_active,
                self.max_applications,
                self.max_applications_per_customer,
                self.custom,
                self.groups,
                self.valid_from,
                self.valid_until,
            )
        )


class DiscountCodePagedQueryResponse(_BaseType):
    #: :class:`int`
    limit: int
    #: :class:`int`
    count: int
    #: Optional :class:`int`
    total: typing.Optional[int]
    #: :class:`int`
    offset: int
    #: List of :class:`commercetools.types.DiscountCode`
    results: typing.Sequence["DiscountCode"]

    def __init__(
        self,
        *,
        limit: int,
        count: int,
        offset: int,
        results: typing.Sequence["DiscountCode"],
        total: typing.Optional[int] = None
    ) -> None:
        self.limit = limit
        self.count = count
        self.total = total
        self.offset = offset
        self.results = results
        super().__init__()

    def __repr__(self) -> str:
        return (
            "DiscountCodePagedQueryResponse(limit=%r, count=%r, total=%r, offset=%r, results=%r)"
            % (self.limit, self.count, self.total, self.offset, self.results)
        )


class DiscountCodeReference(Reference):
    #: Optional :class:`commercetools.types.DiscountCode`
    obj: typing.Optional["DiscountCode"]

    def __init__(self, *, id: str, obj: typing.Optional["DiscountCode"] = None) -> None:
        self.obj = obj
        super().__init__(type_id=ReferenceTypeId.DISCOUNT_CODE, id=id)

    def __repr__(self) -> str:
        return "DiscountCodeReference(type_id=%r, id=%r, obj=%r)" % (
            self.type_id,
            self.id,
            self.obj,
        )


class DiscountCodeResourceIdentifier(ResourceIdentifier):
    def __init__(
        self, *, id: typing.Optional[str] = None, key: typing.Optional[str] = None
    ) -> None:
        super().__init__(type_id=ReferenceTypeId.DISCOUNT_CODE, id=id, key=key)

    def __repr__(self) -> str:
        return "DiscountCodeResourceIdentifier(type_id=%r, id=%r, key=%r)" % (
            self.type_id,
            self.id,
            self.key,
        )


class DiscountCodeUpdate(_BaseType):
    #: :class:`int`
    version: int
    #: :class:`list`
    actions: list

    def __init__(self, *, version: int, actions: list) -> None:
        self.version = version
        self.actions = actions
        super().__init__()

    def __repr__(self) -> str:
        return "DiscountCodeUpdate(version=%r, actions=%r)" % (
            self.version,
            self.actions,
        )


class DiscountCodeUpdateAction(_BaseType):
    #: :class:`str`
    action: str

    def __init__(self, *, action: str) -> None:
        self.action = action
        super().__init__()

    def __repr__(self) -> str:
        return "DiscountCodeUpdateAction(action=%r)" % (self.action,)


class DiscountCodeChangeCartDiscountsAction(DiscountCodeUpdateAction):
    #: List of :class:`commercetools.types.CartDiscountResourceIdentifier` `(Named` ``cartDiscounts`` `in Commercetools)`
    cart_discounts: typing.List["CartDiscountResourceIdentifier"]

    def __init__(
        self, *, cart_discounts: typing.List["CartDiscountResourceIdentifier"]
    ) -> None:
        self.cart_discounts = cart_discounts
        super().__init__(action="changeCartDiscounts")

    def __repr__(self) -> str:
        return "DiscountCodeChangeCartDiscountsAction(action=%r, cart_discounts=%r)" % (
            self.action,
            self.cart_discounts,
        )


class DiscountCodeChangeGroupsAction(DiscountCodeUpdateAction):
    #: List of :class:`str`
    groups: typing.List[str]

    def __init__(self, *, groups: typing.List[str]) -> None:
        self.groups = groups
        super().__init__(action="changeGroups")

    def __repr__(self) -> str:
        return "DiscountCodeChangeGroupsAction(action=%r, groups=%r)" % (
            self.action,
            self.groups,
        )


class DiscountCodeChangeIsActiveAction(DiscountCodeUpdateAction):
    #: :class:`bool` `(Named` ``isActive`` `in Commercetools)`
    is_active: bool

    def __init__(self, *, is_active: bool) -> None:
        self.is_active = is_active
        super().__init__(action="changeIsActive")

    def __repr__(self) -> str:
        return "DiscountCodeChangeIsActiveAction(action=%r, is_active=%r)" % (
            self.action,
            self.is_active,
        )


class DiscountCodeSetCartPredicateAction(DiscountCodeUpdateAction):
    #: Optional :class:`str` `(Named` ``cartPredicate`` `in Commercetools)`
    cart_predicate: typing.Optional[str]

    def __init__(self, *, cart_predicate: typing.Optional[str] = None) -> None:
        self.cart_predicate = cart_predicate
        super().__init__(action="setCartPredicate")

    def __repr__(self) -> str:
        return "DiscountCodeSetCartPredicateAction(action=%r, cart_predicate=%r)" % (
            self.action,
            self.cart_predicate,
        )


class DiscountCodeSetCustomFieldAction(DiscountCodeUpdateAction):
    #: :class:`str`
    name: str
    #: Optional :class:`typing.Any`
    value: typing.Optional[typing.Any]

    def __init__(self, *, name: str, value: typing.Optional[typing.Any] = None) -> None:
        self.name = name
        self.value = value
        super().__init__(action="setCustomField")

    def __repr__(self) -> str:
        return "DiscountCodeSetCustomFieldAction(action=%r, name=%r, value=%r)" % (
            self.action,
            self.name,
            self.value,
        )


class DiscountCodeSetCustomTypeAction(DiscountCodeUpdateAction):
    #: Optional :class:`commercetools.types.TypeResourceIdentifier`
    type: typing.Optional["TypeResourceIdentifier"]
    #: Optional :class:`commercetools.types.FieldContainer`
    fields: typing.Optional["FieldContainer"]

    def __init__(
        self,
        *,
        type: typing.Optional["TypeResourceIdentifier"] = None,
        fields: typing.Optional["FieldContainer"] = None
    ) -> None:
        self.type = type
        self.fields = fields
        super().__init__(action="setCustomType")

    def __repr__(self) -> str:
        return "DiscountCodeSetCustomTypeAction(action=%r, type=%r, fields=%r)" % (
            self.action,
            self.type,
            self.fields,
        )


class DiscountCodeSetDescriptionAction(DiscountCodeUpdateAction):
    #: Optional :class:`commercetools.types.LocalizedString`
    description: typing.Optional["LocalizedString"]

    def __init__(
        self, *, description: typing.Optional["LocalizedString"] = None
    ) -> None:
        self.description = description
        super().__init__(action="setDescription")

    def __repr__(self) -> str:
        return "DiscountCodeSetDescriptionAction(action=%r, description=%r)" % (
            self.action,
            self.description,
        )


class DiscountCodeSetMaxApplicationsAction(DiscountCodeUpdateAction):
    #: Optional :class:`int` `(Named` ``maxApplications`` `in Commercetools)`
    max_applications: typing.Optional[int]

    def __init__(self, *, max_applications: typing.Optional[int] = None) -> None:
        self.max_applications = max_applications
        super().__init__(action="setMaxApplications")

    def __repr__(self) -> str:
        return (
            "DiscountCodeSetMaxApplicationsAction(action=%r, max_applications=%r)"
            % (self.action, self.max_applications)
        )


class DiscountCodeSetMaxApplicationsPerCustomerAction(DiscountCodeUpdateAction):
    #: Optional :class:`int` `(Named` ``maxApplicationsPerCustomer`` `in Commercetools)`
    max_applications_per_customer: typing.Optional[int]

    def __init__(
        self, *, max_applications_per_customer: typing.Optional[int] = None
    ) -> None:
        self.max_applications_per_customer = max_applications_per_customer
        super().__init__(action="setMaxApplicationsPerCustomer")

    def __repr__(self) -> str:
        return (
            "DiscountCodeSetMaxApplicationsPerCustomerAction(action=%r, max_applications_per_customer=%r)"
            % (self.action, self.max_applications_per_customer)
        )


class DiscountCodeSetNameAction(DiscountCodeUpdateAction):
    #: Optional :class:`commercetools.types.LocalizedString`
    name: typing.Optional["LocalizedString"]

    def __init__(self, *, name: typing.Optional["LocalizedString"] = None) -> None:
        self.name = name
        super().__init__(action="setName")

    def __repr__(self) -> str:
        return "DiscountCodeSetNameAction(action=%r, name=%r)" % (
            self.action,
            self.name,
        )


class DiscountCodeSetValidFromAction(DiscountCodeUpdateAction):
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]

    def __init__(
        self, *, valid_from: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.valid_from = valid_from
        super().__init__(action="setValidFrom")

    def __repr__(self) -> str:
        return "DiscountCodeSetValidFromAction(action=%r, valid_from=%r)" % (
            self.action,
            self.valid_from,
        )


class DiscountCodeSetValidFromAndUntilAction(DiscountCodeUpdateAction):
    #: Optional :class:`datetime.datetime` `(Named` ``validFrom`` `in Commercetools)`
    valid_from: typing.Optional[datetime.datetime]
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self,
        *,
        valid_from: typing.Optional[datetime.datetime] = None,
        valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.valid_from = valid_from
        self.valid_until = valid_until
        super().__init__(action="setValidFromAndUntil")

    def __repr__(self) -> str:
        return (
            "DiscountCodeSetValidFromAndUntilAction(action=%r, valid_from=%r, valid_until=%r)"
            % (self.action, self.valid_from, self.valid_until)
        )


class DiscountCodeSetValidUntilAction(DiscountCodeUpdateAction):
    #: Optional :class:`datetime.datetime` `(Named` ``validUntil`` `in Commercetools)`
    valid_until: typing.Optional[datetime.datetime]

    def __init__(
        self, *, valid_until: typing.Optional[datetime.datetime] = None
    ) -> None:
        self.valid_until = valid_until
        super().__init__(action="setValidUntil")

    def __repr__(self) -> str:
        return "DiscountCodeSetValidUntilAction(action=%r, valid_until=%r)" % (
            self.action,
            self.valid_until,
        )



# DO NOT EDIT! This file is automatically generated
class _BaseType:
    def __eq__(self, other):
        if other.__class__ is self.__class__:
            return self.__values__() == other.__values__()
        else:
            return NotImplemented

    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return NotImplemented
        else:
            return not result

    def __lt__(self, other):
        if other.__class__ is self.__class__:
            return self.__values__() < other.__values__()
        else:
            return NotImplemented

    def __le__(self, other):
        if other.__class__ is self.__class__:
            return self.__values__() <= other.__values__()
        else:
            return NotImplemented

    def __gt__(self, other):
        if other.__class__ is self.__class__:
            return self.__values__() > other.__values__()
        else:
            return NotImplemented

    def __ge__(self, other):
        if other.__class__ is self.__class__:
            return self.__values__() >= other.__values__()
        else:
            return NotImplemented

    def __values__(self):
        return tuple(self.__dict__.values())

    def __hash__(self):
        return hash((self.__class__,) + self.__values__())



import typing

from commercetools.types import ErrorResponse


class CommercetoolsError(Exception):
    response: ErrorResponse
    correlation_id: typing.Optional[str]

    def __init__(
        self,
        message: typing.Any,
        errors: typing.List[dict],
        response: ErrorResponse,
        correlation_id: str = None,
    ) -> None:
        super().__init__(message)
        self.response = response
        self.errors = errors
        self.correlation_id = correlation_id

    def __str__(self):
        result = super().__str__()
        if self.details:
            return f"{result} ({', '.join(self.details)})"
        return result

    @property
    def details(self) -> typing.List[str]:
        return [
            e["detailedErrorMessage"]
            for e in self.errors
            if "detailedErrorMessage" in e
        ]

    @property
    def codes(self) -> typing.List[str]:
        try:
            if self.response.errors is not None:
                return [e.code for e in self.response.errors]
            else:
                return []
        except AttributeError:
            return []

    @property
    def code(self) -> str:
        """Convenience property to easily get the error code.

        Returns the code of the first error, just as
        'message' is always the message of the first error.
        """
        try:
            return self.codes[0]
        except KeyError:
            return ""



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types

__all__ = [
    "ApiClientDraftSchema",
    "ApiClientPagedQueryResponseSchema",
    "ApiClientSchema",
]


class ApiClientDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ApiClientDraft`."""

    name = marshmallow.fields.String(allow_none=True)
    scope = marshmallow.fields.String(allow_none=True)
    delete_days_after_creation = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="deleteDaysAfterCreation"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ApiClientDraft(**data)


class ApiClientPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ApiClientPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._api_client.ApiClientSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ApiClientPagedQueryResponse(**data)


class ApiClientSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ApiClient`."""

    id = marshmallow.fields.String(allow_none=True)
    name = marshmallow.fields.String(allow_none=True)
    scope = marshmallow.fields.String(allow_none=True)
    created_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="createdAt"
    )
    last_used_at = marshmallow.fields.Date(
        allow_none=True, missing=None, data_key="lastUsedAt"
    )
    delete_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="deleteAt"
    )
    secret = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ApiClient(**data)



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)

__all__ = [
    "ProductDiscountChangeIsActiveActionSchema",
    "ProductDiscountChangeNameActionSchema",
    "ProductDiscountChangePredicateActionSchema",
    "ProductDiscountChangeSortOrderActionSchema",
    "ProductDiscountChangeValueActionSchema",
    "ProductDiscountDraftSchema",
    "ProductDiscountMatchQuerySchema",
    "ProductDiscountPagedQueryResponseSchema",
    "ProductDiscountReferenceSchema",
    "ProductDiscountResourceIdentifierSchema",
    "ProductDiscountSchema",
    "ProductDiscountSetDescriptionActionSchema",
    "ProductDiscountSetKeyActionSchema",
    "ProductDiscountSetValidFromActionSchema",
    "ProductDiscountSetValidFromAndUntilActionSchema",
    "ProductDiscountSetValidUntilActionSchema",
    "ProductDiscountUpdateActionSchema",
    "ProductDiscountUpdateSchema",
    "ProductDiscountValueAbsoluteDraftSchema",
    "ProductDiscountValueAbsoluteSchema",
    "ProductDiscountValueDraftSchema",
    "ProductDiscountValueExternalDraftSchema",
    "ProductDiscountValueExternalSchema",
    "ProductDiscountValueRelativeDraftSchema",
    "ProductDiscountValueRelativeSchema",
    "ProductDiscountValueSchema",
]


class ProductDiscountDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountDraft`."""

    name = LocalizedStringField(allow_none=True)
    key = marshmallow.fields.String(allow_none=True, missing=None)
    description = LocalizedStringField(allow_none=True, missing=None)
    value = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "absolute": "commercetools._schemas._product_discount.ProductDiscountValueAbsoluteDraftSchema",
            "external": "commercetools._schemas._product_discount.ProductDiscountValueExternalDraftSchema",
            "relative": "commercetools._schemas._product_discount.ProductDiscountValueRelativeDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    predicate = marshmallow.fields.String(allow_none=True)
    sort_order = marshmallow.fields.String(allow_none=True, data_key="sortOrder")
    is_active = marshmallow.fields.Bool(allow_none=True, data_key="isActive")
    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )
    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ProductDiscountDraft(**data)


class ProductDiscountMatchQuerySchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountMatchQuery`."""

    product_id = marshmallow.fields.String(allow_none=True, data_key="productId")
    variant_id = marshmallow.fields.Integer(allow_none=True, data_key="variantId")
    staged = marshmallow.fields.Bool(allow_none=True)
    price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.QueryPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ProductDiscountMatchQuery(**data)


class ProductDiscountPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._product_discount.ProductDiscountSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ProductDiscountPagedQueryResponse(**data)


class ProductDiscountReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._product_discount.ProductDiscountSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ProductDiscountReference(**data)


class ProductDiscountResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.ProductDiscountResourceIdentifier(**data)


class ProductDiscountSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscount`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    name = LocalizedStringField(allow_none=True)
    key = marshmallow.fields.String(allow_none=True, missing=None)
    description = LocalizedStringField(allow_none=True, missing=None)
    value = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "absolute": "commercetools._schemas._product_discount.ProductDiscountValueAbsoluteSchema",
            "external": "commercetools._schemas._product_discount.ProductDiscountValueExternalSchema",
            "relative": "commercetools._schemas._product_discount.ProductDiscountValueRelativeSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    predicate = marshmallow.fields.String(allow_none=True)
    sort_order = marshmallow.fields.String(allow_none=True, data_key="sortOrder")
    is_active = marshmallow.fields.Bool(allow_none=True, data_key="isActive")
    references = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("typeId", "type_id"),
            discriminator_schemas={
                "cart-discount": "commercetools._schemas._cart_discount.CartDiscountReferenceSchema",
                "cart": "commercetools._schemas._cart.CartReferenceSchema",
                "category": "commercetools._schemas._category.CategoryReferenceSchema",
                "channel": "commercetools._schemas._channel.ChannelReferenceSchema",
                "key-value-document": "commercetools._schemas._custom_object.CustomObjectReferenceSchema",
                "customer-group": "commercetools._schemas._customer_group.CustomerGroupReferenceSchema",
                "customer": "commercetools._schemas._customer.CustomerReferenceSchema",
                "discount-code": "commercetools._schemas._discount_code.DiscountCodeReferenceSchema",
                "inventory-entry": "commercetools._schemas._inventory.InventoryEntryReferenceSchema",
                "order-edit": "commercetools._schemas._order_edit.OrderEditReferenceSchema",
                "order": "commercetools._schemas._order.OrderReferenceSchema",
                "payment": "commercetools._schemas._payment.PaymentReferenceSchema",
                "product-discount": "commercetools._schemas._product_discount.ProductDiscountReferenceSchema",
                "product-type": "commercetools._schemas._product_type.ProductTypeReferenceSchema",
                "product": "commercetools._schemas._product.ProductReferenceSchema",
                "review": "commercetools._schemas._review.ReviewReferenceSchema",
                "shipping-method": "commercetools._schemas._shipping_method.ShippingMethodReferenceSchema",
                "shopping-list": "commercetools._schemas._shopping_list.ShoppingListReferenceSchema",
                "state": "commercetools._schemas._state.StateReferenceSchema",
                "store": "commercetools._schemas._store.StoreReferenceSchema",
                "tax-category": "commercetools._schemas._tax_category.TaxCategoryReferenceSchema",
                "type": "commercetools._schemas._type.TypeReferenceSchema",
                "zone": "commercetools._schemas._zone.ZoneReferenceSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        )
    )
    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )
    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ProductDiscount(**data)


class ProductDiscountUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountUpdateAction(**data)


class ProductDiscountUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "changeIsActive": "commercetools._schemas._product_discount.ProductDiscountChangeIsActiveActionSchema",
                "changeName": "commercetools._schemas._product_discount.ProductDiscountChangeNameActionSchema",
                "changePredicate": "commercetools._schemas._product_discount.ProductDiscountChangePredicateActionSchema",
                "changeSortOrder": "commercetools._schemas._product_discount.ProductDiscountChangeSortOrderActionSchema",
                "changeValue": "commercetools._schemas._product_discount.ProductDiscountChangeValueActionSchema",
                "setDescription": "commercetools._schemas._product_discount.ProductDiscountSetDescriptionActionSchema",
                "setKey": "commercetools._schemas._product_discount.ProductDiscountSetKeyActionSchema",
                "setValidFrom": "commercetools._schemas._product_discount.ProductDiscountSetValidFromActionSchema",
                "setValidFromAndUntil": "commercetools._schemas._product_discount.ProductDiscountSetValidFromAndUntilActionSchema",
                "setValidUntil": "commercetools._schemas._product_discount.ProductDiscountSetValidUntilActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.ProductDiscountUpdate(**data)


class ProductDiscountValueDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValueDraft`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValueDraft(**data)


class ProductDiscountValueSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValue`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValue(**data)


class ProductDiscountChangeIsActiveActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountChangeIsActiveAction`."""

    is_active = marshmallow.fields.Bool(allow_none=True, data_key="isActive")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountChangeIsActiveAction(**data)


class ProductDiscountChangeNameActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountChangeNameAction`."""

    name = LocalizedStringField(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountChangeNameAction(**data)


class ProductDiscountChangePredicateActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountChangePredicateAction`."""

    predicate = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountChangePredicateAction(**data)


class ProductDiscountChangeSortOrderActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountChangeSortOrderAction`."""

    sort_order = marshmallow.fields.String(allow_none=True, data_key="sortOrder")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountChangeSortOrderAction(**data)


class ProductDiscountChangeValueActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountChangeValueAction`."""

    value = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "absolute": "commercetools._schemas._product_discount.ProductDiscountValueAbsoluteDraftSchema",
            "external": "commercetools._schemas._product_discount.ProductDiscountValueExternalDraftSchema",
            "relative": "commercetools._schemas._product_discount.ProductDiscountValueRelativeDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountChangeValueAction(**data)


class ProductDiscountSetDescriptionActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountSetDescriptionAction`."""

    description = LocalizedStringField(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountSetDescriptionAction(**data)


class ProductDiscountSetKeyActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountSetKeyAction(**data)


class ProductDiscountSetValidFromActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountSetValidFromAction`."""

    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountSetValidFromAction(**data)


class ProductDiscountSetValidFromAndUntilActionSchema(
    ProductDiscountUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountSetValidFromAndUntilAction`."""

    valid_from = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validFrom"
    )
    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountSetValidFromAndUntilAction(**data)


class ProductDiscountSetValidUntilActionSchema(ProductDiscountUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountSetValidUntilAction`."""

    valid_until = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="validUntil"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.ProductDiscountSetValidUntilAction(**data)


class ProductDiscountValueAbsoluteDraftSchema(ProductDiscountValueDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValueAbsoluteDraft`."""

    money = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValueAbsoluteDraft(**data)


class ProductDiscountValueAbsoluteSchema(ProductDiscountValueSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValueAbsolute`."""

    money = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("type", "type"),
            discriminator_schemas={
                "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
                "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        )
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValueAbsolute(**data)


class ProductDiscountValueExternalDraftSchema(ProductDiscountValueDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValueExternalDraft`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValueExternalDraft()


class ProductDiscountValueExternalSchema(ProductDiscountValueSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValueExternal`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValueExternal()


class ProductDiscountValueRelativeDraftSchema(ProductDiscountValueDraftSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValueRelativeDraft`."""

    permyriad = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValueRelativeDraft(**data)


class ProductDiscountValueRelativeSchema(ProductDiscountValueSchema):
    """Marshmallow schema for :class:`commercetools.types.ProductDiscountValueRelative`."""

    permyriad = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.ProductDiscountValueRelative(**data)
